# -*- coding: utf-8 -*-
# Docbook XML markup check

from xml.parsers.expat import ParserCreate
from xml.parsers.expat import ExpatError
from KPC.classes import Error, BaseCheck

# 메시지에 대해 XML 파서를 돌려 well formed인지 검사하고, 태그가 알려진
# DocBook 태그인지 검사한다.

# DocBook XML의 태그는 수백 가지에 달하지만, 결벽증 마냥 모든 걸 나열할
# 필요는 없고 실제 발견될 때마다 추가하도록 한다.

known_db_tags = set([
    'acronym',
    'address',
    'anchor',
    'application',
    'citerefentry',
    'citetitle',
    'city',
    'classname',
    'command',
    'computeroutput',
    'country',
    'email',
    'emphasis',
    'entry',
    'envar',
    'filename',
    'firstterm',
    'footnote',
    'footnoteref',
    'glossterm',
    'guibutton',
    'guiicon',
    'guilabel',
    'guimenu',
    'guimenuitem',
    'guisubmenu',
    'indexterm',
    'informalexample',
    'informaltable',
    'interface',
    'interfacename',
    'itemizedlist',
    'keycap',
    'keycombo',
    'keysym',
    'link',
    'listitem',
    'literal',
    'manvolnum',
    'menuchoice',
    'option',
    'orderedlist',
    'para',
    'phrase',
    'postcode',
    'primary',
    'prompt',
    'quote',
    'refentrytitle',
    'remark',
    'replaceable',
    'row',
    'screen',
    'screen',
    'secondary',
    'see',
    'sgmltag',
    'shortcut',
    'state',
    'street',
    'subscript',
    'superscript',
    'systemitem',
    'tbody',
    'term',
    'tertiary',
    'tgroup',
    'thead',
    'title',
    'trademark',
    'ulink',
    'uri',
    'userinput',
    'variablelist',
    'varlistentry',
    'varname',
    'xref'
])

tag_error_string = 'XML 태그의 짝이 맞지 않습니다'
notdb_error_string = '<%s>: 알려진 DocBook 태그가 아닙니다'


class NotDocBook(Exception):
    pass


def check_db_tags(name):
    if name == 'KPC_DummyTag':
        pass
    elif name[:12] == 'placeholder-':
        # old gnome-doc-utils magic
        pass
    elif name[:7] == '_:item-' or name[:7] == '_:link-':
        # newer gnome-doc-utils magic
        pass
    elif not name in known_db_tags:
        raise NotDocBook(name)


def start_element(name, attr):
    check_db_tags(name)


def end_element(name):
    check_db_tags(name)


class MarkupDocbookCheck(BaseCheck):
    def check(self, entry, context):
        if not entry.references or not '.xml:' in entry.references[0]:
            # not from an XML file
            return []
        msgid = entry.msgid
        msgstr = entry.msgstr
        if msgid == 'translator-credits':  # gnome-doc-utils magic
            return []
        parser = ParserCreate()
        parser.StartElementHandler = start_element
        parser.EndElementHandler = end_element
        parser.UseForeignDTD(True)
        try:
            parser.Parse('<KPC_DummyTag>' + msgstr + '</KPC_DummyTag>')
        except ExpatError as e:
            return [Error(tag_error_string)]
        except NotDocBook as e:
            return [Error(notdb_error_string % e)]
        return []

name = 'convention/markup-docbook'
description = 'DocBook 마크업의 짝이 맞는지 검사합니다'
checker = MarkupDocbookCheck()



# -*- coding: utf-8 -*-
# Mallard XML markup check

from xml.parsers.expat import ParserCreate
from xml.parsers.expat import ExpatError
from KPC.classes import Error, BaseCheck

# 메시지에 대해 XML 파서를 돌려 well formed인지 검사하고, 태그가 알려진
# Mallard 태그인지 검사한다.
# grep 'element name' mallard-1.0.rng | cut -f 2 -d'"'
known_tags = set([
    'page',
    'section',
    'info',
    'credit',
    'link',
    'title',
    'license',
    'desc',
    'revision',
    'years',
    'email',
    'name',
    'links',
    'code',
    'example',
    'media',
    'p',
    'screen',
    'quote',
    'comment',
    'figure',
    'listing',
    'note',
    'synopsis',
    'list',
    'item',
    'steps',
    'item',
    'terms',
    'item',
    'tree',
    'item',
    'table',
    'col',
    'colgroup',
    'tr',
    'thead',
    'tbody',
    'tfoot',
    'td',
    'title',
    'subtitle',
    'desc',
    'cite',
    'app',
    'code',
    'cmd',
    'output',
    'em',
    'file',
    'gui',
    'guiseq',
    'link',
    'media',
    'keyseq',
    'key',
    'span',
    'sys',
    'input',
    'var',
])

tag_error_string = 'XML 태그의 짝이 맞지 않습니다'
unknown_tag_error_string = '<%s>: 알려진 Mallard 태그가 아닙니다'


class UnknownTag(Exception):
    pass


def check_tags(name):
    if name == 'KPC_DummyTag':
        pass
    elif name[:12] == 'placeholder-':
        # old gnome-doc-utils magic
        pass
    elif name[:7] == '_:item-' or name[:7] == '_:link-':
        # newer gnome-doc-utils magic
        pass
    elif not name in known_tags:
        raise UnknownTag(name)


def start_element(name, attr):
    check_tags(name)


def end_element(name):
    check_tags(name)


class MarkupMallardCheck(BaseCheck):
    def check(self, entry, context):
        if not entry.references or not '.page:' in entry.references[0]:
            # not from an XML file
            return []
        msgid = entry.msgid
        msgstr = entry.msgstr
        if msgid == 'translator-credits':  # gnome-doc-utils magic
            return []
        parser = ParserCreate()
        parser.StartElementHandler = start_element
        parser.EndElementHandler = end_element
        parser.UseForeignDTD(True)
        try:
            parser.Parse('<KPC_DummyTag>' + msgstr + '</KPC_DummyTag>')
        except ExpatError as e:
            return [Error(tag_error_string)]
        except UnknownTag as e:
            return [Error(unknown_tag_error_string % e)]
        return []

name = 'convention/markup-mallard'
description = 'Mallard 마크업의 짝이 맞는지 검사합니다'
checker = MarkupMallardCheck()



# -*- coding: utf-8 -*-

# 그놈 프로그램 gschema에서 많이 쓰이는 메시지 통일

from KPC.classes import Error, BaseCheck

data = [
    {'msgid': 'Window maximized',
     'msgstr': '창 최대화'},
    {'msgid': 'Window maximized state.',
     'msgstr': '창 최대화 상태.'},
    {'msgid': 'Window size',
     'msgstr': '창 크기'},
    {'msgid': 'Window size (width and height).',
     'msgstr': '창 크기(너비 및 높이).'},
    {'msgid': 'Window position.',
     'msgstr': '창 위치'},
    {'msgid': 'Window position (x and y).',
     'msgstr': '창 위치(가로 및 세로).'},
]


class GschemaCheck(BaseCheck):
    errstr = '일관성을 위해 다음과 같이 번역합니다: %s'

    def check(self, entry, context):
        if not True in [('.gschema' in p) for p in entry.references]:
            return []
        for d in data:
            if entry.msgid.lower() != d['msgid'].lower():
                continue
            if entry.msgstr != d['msgstr']:
                return [Error(self.errstr % d['msgstr'])]
        return []

name = 'terminology/gschema'
description = 'gschema에서 자주 사용하는 메시지의 번역이 통일된 번역인지 검사합니다'
checker = GschemaCheck()



# -*- coding: utf-8 -*-

import re
from KPC.classes import Error, BaseCheck

josa_data = [('가', '이', '이(가)'),
             ('를', '을', '을(를)'),
             ('는', '은', '은(는)'),
             ('와', '과', '과(와)'),
             ('로', '으로', '(으)로'),
             ('로서', '으로서', '(으)로서'),
             ('로써', '으로써', '(으)로써'),
             ('로부터', '으로부터', '(으)로부터'),
             ('라는', '이라는', '(이)라는')]
c_format = '%(?:[1-9][0-9]*\$)?[-+ #\'0]*(?:[1-9][0-9]*)?(?:\.[0-9]+)?' \
           '(?:(?:hh|h|j|l|L|ll|q|t|z|Z)?' \
           '[dioufeEgGaAcCspnmhjlLqtxXzZ1-9]|hh|ll)'
py_format = '%(?:\([A-Za-z]\w+\))?[#-0\ +]*(?:[0-9]+|\*)?(?:\.[0-9]+)?' \
            '[hlL]?[diouxXeEfFgGcrs]'

josa = '(?:' + '|'.join([p[0]+'|'+p[1] for p in josa_data]) + ')'
josa_c_re = re.compile('(?P<case>(?P<fmt>'+c_format+'[\'\"]?) ?'
                       '(?P<josa>'+josa+'))(?:\s|$)')
josa_py_re = re.compile('(?P<case>(?P<fmt>'+py_format+'[\'\"]?) ?'
                        '(?P<josa>'+josa+'))(?:\s|$)')


def josa_suggest(cho):
    for (a, b, s) in josa_data:
        if cho == a or cho == b:
            return s
    return '????'


class JosaAlternativeCheck(BaseCheck):
    errstr = '\"%s\": 받침에 따른 조사 구별이 없습니다. \'%s\''

    def check(self, entry, context):
        if entry.is_c_format():
            josa_re = josa_c_re
        elif entry.is_python_format():
            josa_re = josa_py_re
        else:
            return []
        msgstr = entry.msgstr
        errors = []
        while True:
            mo = josa_re.search(msgstr)
            if mo:
                sug = mo.group('fmt') + josa_suggest(mo.group('josa'))
                errors.append(Error(self.errstr % (mo.group('case'), sug)))
                msgstr = msgstr[mo.end():]
            else:
                break
        return errors

name = 'language/josa-alternative'
description = '조사 구분을 했는지 검사합니다'
checker = JosaAlternativeCheck()



#!/usr/bin/env python

import sys
if sys.version_info.major < 3:
    print('This program is only for Python 3.x. Run with python3.')
    sys.exit(1)

from distutils.core import setup
from KPC.config import VERSION

setup(name='ko-po-check',
      version=VERSION,
      author='Changwoo Ryu',
      author_email='cwryu@debian.org',
      url='https://github.com/changwoo/ko-po-check',
      license='GPLv3 or later',
      description='Korean PO checker',
      long_description='Korean message translation checker',
      platforms=['any'],
      download_url='https://github.com/changwoo/ko-po-check/releases',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: ' +
          'GNU General Public License v3 or later (GPLv3+)',
          'Natural Language :: Korean',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3',
          'Topic :: Software Development :: Localization',
      ],

      packages=[
          'KPC', 'KPC/checks',
          'KPC/checks/convention',
          'KPC/checks/language',
          'KPC/checks/terminology'
      ],
      scripts=['scripts/ko-po-check'])



# -*- coding: utf-8 -*-

import re
from KPC.classes import Error, BaseCheck

err_fmt = '검색어 번역은 \';\'으로 끝나야 합니다'
err_org = '%s: 검색어 번역에는 원문 키워드를 포함해야 합니다'
err_dup = '%s: 검색어가 중복되어 있습니다'

re_keyword = re.compile(r'^(?:[\w ]+;)+$')


class DesktopKeywordCheck(BaseCheck):
    def is_keyword(self, entry):
        if not True in [('.desktop' in p) for p in entry.references]:
            return False
        return re_keyword.match(entry.msgid)

    def check(self, entry, context):
        if not self.is_keyword(entry):
            return []
        msgid_keywords = entry.msgid[:-1].split(';')
        if entry.msgstr[-1] != ';':
            return [Error(err_fmt)]
        msgstr_keywords = entry.msgstr[:-1].split(';')
        for k in msgid_keywords:
            if not k in msgstr_keywords:
                return [Error(err_org % k)]
        s = set()
        for k in msgstr_keywords:
            if k in s:
                return [Error(err_dup % k)]
            s.add(k)
        return []


name = 'convention/desktop-keyword'
description = 'freedesktop.org desktop 엔트리의 Keyword 번역을 확인합니다'
checker = DesktopKeywordCheck()
